﻿package com.microsoft {
	
	import flash.data.SQLStatement;
	import flash.errors.SQLError;
	import flash.events.EventDispatcher;
	import flash.data.SQLConnection;
	import flash.filesystem.File;
	import flash.filesystem.FileStream;
	import flash.net.URLLoader;
	import flash.events.Event;
	import flash.events.IOErrorEvent;
	import flash.events.SecurityErrorEvent;
	import flash.net.URLRequest;
	import flash.net.URLVariables;
	import flash.utils.Timer;
	import flash.events.TimerEvent;
	import flash.events.HTTPStatusEvent;
	
	public class ServerDatabase extends EventDispatcher{
		
		private var base:String = "https://teched.satelliteapps.co.nz/st/api/stapi.php";
		private var key:String = "JO6aqzOlkcPZvdPa1mA5";// massey
		
		private var conn:SQLConnection = new SQLConnection(); 
		private var folder:File = File.applicationStorageDirectory; 
		private var dbFile:File = folder.resolvePath("scans.db");
		
		private var notSents:Array;
		private var curr:int = 0;
		
		private var fs:FileStream = new FileStream();
		private var rfids:Object;
		private var rfidFile:File = folder.resolvePath("rfids.json");
		
		private var _netPingTimer:Timer;
		private var fields:String = "rfid";
		
		public var lastUpdate:Date = null;
		
		public var session:int = 0;
		
		public function ServerDatabase(fields:String = "rfid") {
			// constructor code
			
			this.fields = fields;
			
			initSQLdbase();
			
			_netPingTimer = new Timer(120000); //600000 every 10 minutes
			_netPingTimer.start();
			_netPingTimer.addEventListener(TimerEvent.TIMER, netPing, false, 0, true);
			netPing();
		}
		
		// poll the server to keep this alive
		private function netPing(e:TimerEvent = null):void{
			
			trace("Update local rfid list. Send up any new scans.");
			updateNotSents();
		}
		
		public function checkLocalRFID(rfid:String):Boolean{
			var result:Boolean = false;
			if(rfids){
				
				var l:int = rfids["payload"]["table_data"].length;
				for(var i:int = 0; i < l; i++){
					//trace(rfids["payload"]["table_data"][i]["rfid"]);
					if(rfids["payload"]["table_data"][i]["rfid"] == rfid){
						result = true;
						break;
					}
				}
			}
			return result;
		}
		
		public function getVar(rfid:String, varName:String):*{
			var result = "";
			var l:int = rfids["payload"]["table_data"].length;
			for(var i:int = 0; i < l; i++){
				//trace(rfids["payload"]["table_data"][i]["rfid"]);
				if(rfids["payload"]["table_data"][i]["rfid"] == rfid && rfids["payload"]["table_data"][i][varName] != 0){
					result = rfids["payload"]["table_data"][i][varName];
					break;
				}
			}
			return result;
		}
		
		public function addScan(json:String, now:Boolean = false):void{
			var insertData:SQLStatement = new SQLStatement();
			var result:Date = new Date();
			var retime:Number = result.time;			
			insertData.sqlConnection = conn;
			insertData.text = "INSERT INTO scans (json, timestamp, sent) VALUES ('" + escape(json) + "', '" + retime + "', 0)";
			
			try { 
				insertData.execute();				
			}catch (error:SQLError) { 
				trace("Error message:", error.message); 
				trace("Details:", error.details); 
			}
			
			if(now){
				updateNotSents();
			}
		}
		
		private function initSQLdbase():void{
			try{ 
				conn.open(dbFile); 
				trace("the database was created successfully"); 
				var createStmt:SQLStatement = new SQLStatement(); 
				createStmt.sqlConnection = conn; 
					 
				var sql:String =  
					"CREATE TABLE IF NOT EXISTS scans (" +  
					"    id INTEGER PRIMARY KEY AUTOINCREMENT, " +  
					"    json TEXT, " +  
					"    timestamp INTEGER," +  
					"    sent INTEGER)"; 
				createStmt.text = sql; 
					 
				try 
				{ 
					createStmt.execute(); 
					trace("Table created"); 
				} 
				catch (error:SQLError) 
				{ 
					trace("Error message:", error.message); 
					trace("Details:", error.details); 
				} 
				
			} catch (error:SQLError) { 
				trace("Error message:", error.message); 
				trace("Details:", error.details); 
			}
		}
		
		private function updateNotSents():void{
			var checkStatement:SQLStatement = new SQLStatement();
			checkStatement.sqlConnection = conn;
			checkStatement.text = "SELECT * FROM scans WHERE sent = 0";
			try{
				checkStatement.execute();
				curr = 0;
				notSents = checkStatement.getResult().data;
				trace("notSents:",notSents);
				if(notSents){
					sendScan();
				}else{
					trace("No scans. Get latest RFID list.");
					getRFIDList();
				}
				
			} catch (error:SQLError) { 
				trace("Error message:", error.message); 
				trace("Details:", error.details); 
			}
		}
		
		//*
		private function getRFIDList():void{
			//*
			if(fields != null){
				var url:String = base;
				var loader:URLLoader = new URLLoader();
				
				var request:URLRequest = new URLRequest(url);
				request.method = "POST";
				var vars:URLVariables = new URLVariables();
				vars.json =  '{"auth_key":"' + key + '","function":"tb_cache_request","table_name":"delegate", "table_fields":"' + fields + '"}';
				
				request.data = vars;
				
				trace("getRFIDList", vars.json);
				
				loader.addEventListener(Event.COMPLETE, rfidsLoaded, false, 0, true);
				loader.addEventListener(IOErrorEvent.IO_ERROR, ioError, false, 0, true);
				loader.addEventListener(SecurityErrorEvent.SECURITY_ERROR, securityError, false, 0, true);
				loader.load(request);
			}
			//*/
		}
		
		private function rfidsLoaded(e:Event):void{
			//*
			//trace("loaded",e.target.data);
			e.currentTarget.removeEventListener(Event.COMPLETE, rfidsLoaded);
			e.currentTarget.removeEventListener(IOErrorEvent.IO_ERROR, ioError);
			e.currentTarget.removeEventListener(SecurityErrorEvent.SECURITY_ERROR, securityError);
			fs.open(rfidFile, "write");
				fs.writeUTFBytes( e.target.data );
			fs.close();
			//trace( e.target.data);
			fs.open(rfidFile, "read");
				//trace();
				rfids = JSON.parse(fs.readUTFBytes( fs.bytesAvailable ));
			fs.close();
			
			//dispatchEvent(new ArrayEvent("users", new Array({users:rfids["payload"]["table_data"].length})));
			//*/
		}
		
		private function ioError (e:IOErrorEvent):void{
			trace("IO error", e);
			lastUpdate = null;
		}
		
		private function securityError (e:SecurityErrorEvent):void{
			trace("IO error");
			lastUpdate = null;
		}
		
		//*
		private function sendScan():void{
			
			var loader:URLLoader = new URLLoader();
			var request:URLRequest;
			//loader.load(request);
			//trace(">",notSents[curr]["json"].substr(0,5));
			if(notSents[curr]["json"].substr(0,5) == "https"){
				request = new URLRequest(notSents[curr]["json"]);
			}else{
				request = new URLRequest(base);
				request.method = "POST";
				trace(request.url);
				var vars:URLVariables = new URLVariables();
				vars.json =  '{"auth_key":"' + key + '", ' +  unescape(notSents[curr]["json"]) + '}';
				request.data = vars;
				trace(vars.json);
				
			}
			
			loader.addEventListener(Event.COMPLETE, sentOld, false, 0, true);
			loader.addEventListener(HTTPStatusEvent.HTTP_RESPONSE_STATUS, httpStatus, false, 0, true);
			loader.addEventListener(IOErrorEvent.IO_ERROR, ioError, false, 0, true);
			loader.addEventListener(SecurityErrorEvent.SECURITY_ERROR, securityError, false, 0, true);
			loader.load(request);
		}
		
		private function httpStatus(e:HTTPStatusEvent):void{
			trace(e);
		}
		
		private function sentOld(e:Event):void{
			trace("loaded",e.target.data);
			e.currentTarget.removeEventListener(Event.COMPLETE, sentOld);
			e.currentTarget.removeEventListener(IOErrorEvent.IO_ERROR, ioError);
			e.currentTarget.removeEventListener(SecurityErrorEvent.SECURITY_ERROR, securityError);
			
			lastUpdate = new Date();
			
			var updateStatement:SQLStatement = new SQLStatement();
			updateStatement.sqlConnection = conn;
			updateStatement.text = "UPDATE scans SET sent = 1 WHERE timestamp = " + notSents[curr]["timestamp"] + " AND json = '" + notSents[curr]["json"] + "'";
			
			try{
				updateStatement.execute();
				trace("sent");
			} catch (error:SQLError) { 
				trace("Error message:", error.message); 
				trace("Details:", error.details); 
			}
			
			trace("Sent old", curr);
			curr++;
			
			if(curr < notSents.length){
				sendScan();
			}else{
				trace("All updated. Get new RFID list.");
				getRFIDList();
			}
			
		}
		//*/
		
	}
	
}
